<?php

/* ***************************************************************************

  This is index.php, the main file of the book bidding application.

  It does multiple things:
    - first, we define here the location of our config file, the
      e-mail address of the system admin, etc.
    - then, it asks for an HTTP login... This is only to protect
      the application from other students before hand-in date
      (so that they cannot copy the design or any other idea)
    - it then includes basic skin information (CSS) and also
      prints the tables that are the "general look" of the site
      and we also define some functions for formatting the
      inputs (replace dangerous characters) and outputs (format
      things like telephone or ISBN numbers)
    - after, a very important activity: session recognition. This
      file also has the code to recognise a session ID passed
      using HTTP GET protocol, and load the current user's
      data based on the session ID.
    - last of all, it reads the current desired action of the user
      (sent via the "do" variable using HTTP GET protocol) and
      based on the login status includes the PHP file for
      the requested action (or the "log in" page with a redirect)

  This file, alone, is pretty useless; and the other files without this
  file are pretty lost and also vulnerable to attacks (using web forms)
  We therefore implement a simple system so that other pages can *only*
  be called by index.php

**************************************************************************** */

// PHP should only report big problems
error_reporting(E_ERROR);

// We define this so that our external files cannot be called directly
define("bath","university");

// The configuration file path... It has to contain the $dsn variable and include DB.php
define("config_address","db.php");

// Define the prefix for the tables
define("table_prefix","st221_rb247_project_");

// The e-mail address of the system administrator...
// To tell him / her about people sending false book information
define("system_admin","st221@bath.ac.uk; rb247@bath.ac.uk");

/*
// Uncomment this part to restrict access

$auth=0;

if($_SERVER['PHP_AUTH_USER']!="st221" || $_SERVER['PHP_AUTH_PW']!="rb247"){
  header('WWW-Authenticate: Basic realm="Please enter a user name and password"');
  header('HTTP/1.0 401 Unauthorized');
  $auth=1;}

if($auth)
{
  echo "<br><br>Sorry... You need to enter a valid user name and password to access this page!<br><br><br><a href=../>Click here</a> to return to the index</center>";
  die();
}
*/

// We use output buffering, so that out echo statements will echo to a buffer
// instead of the user: this is useful since echo will wait for the data to be
// sent, which can take a while when the user has a slow connection! The bad thing
// is that during all this time, we also keep the connection to the MySQL server
// open!! A buffer write is instant, so the code is really executed rapidly, so
// the connection to the MySQL server will not remain open for more than necessary.
ob_implicit_flush(0);
ob_start();

// We use this function to calculate in how much time out page got generated
function getmicrotime(){ 
  list($usec,$sec)=explode(" ",microtime());
  return ((float)$usec+(float)$sec);}

define('PAGE_START_TIME', getmicrotime());

?>

<html>
<head>
<title>Book auctions</title>

<?php
// Read the CSS file
$handle=fopen("style.css","r");
echo "<style>";
echo fread($handle,filesize("style.css"));
echo "</style>";
fclose($handle);
?>

</head>
<body>

<table width=100% height=100% cellpadding=0 cellspacing=0>
<tr><td valign=top align=center>

<table border="0" cellpadding="0" cellpadding="0" class="BandeBleu" width="100%" height="90"> 
  <tr> 
    <td align="left" valign="middle"></td>
    <td></td> 
    <td align="right"><img src="images/logo-xs-trans.gif"></td> 
  </tr> 
</table>

<?php

// Takes the integer value of a big int (10 digits)

// This function seems to be weird, but actually makes sense: a phone (or ISBN) number
// has 10 digits, which makes 34 bits; and the intval() function in PHP works on 32-bit
// integers! We therefore have to do an imtval for 10 digits

function intval_10d($what)
{
  $head=intval(substr($what,0,6));
  $tail=intval("1".substr($what,6,4));
  $tail=substr("$tail",1,4);
  {
    return $head.$tail;
  }
}

// Reads post data and does some basic reformatting

// If length is 0, then we're trying to read an integer
// If length is negative, then we're trying to read a date
// Default length is 32

function read_p( $what , $length=32 )
{
  $what = $_POST[$what];
  if( $length == 0 )
  {
    return intval_10d($what);
  }
  elseif($length<0)
  {
    // length is negative, so it is a date
    // I'm actually not sure we use this ...
    $what=explode(".",$what);
    return mktime(0,0,0,$what[1],$what[0],$what[2]);
  }
  else
  {
    // length is positive, so read the first length characters of the input
    $what=substr(str_replace(array("<","\\","\""),array("&lt;","","'"),$what),0,$length);
    if(strlen($what)<1)
    {
      $what=false;
    }
    return $what;
  }
}

// Transforms a telephone number integer into the "usual" format (0 xxxx xxx xxx)

function telephone( $number )
{
  // int -> string
  $number = "$number";

  return "0 ".substr($number,0,4)." ".substr($number,4,3)." ".substr($number,7,11);
}

// Transforms an ISBN number integer into the "usual" format (x-xxx-xxxxx-x)

function isbn($number)
{
  // int -> string
  $number="$number";

  // Add 0s at the beginning
  while(strlen($number)<10)
  {
    $number="0".$number;
  }

  return substr($number,0,1)."-".substr($number,1,3)."-".substr($number,4,5)."-".substr($number,9,1);
}

// Function to create session IDs

function superRand()
{
  $r=0;$t=time();
  for($i=0;$i<11;$i++)
    $r=$r*10+rand(0,9);
  return md5(strtolower(base_convert($t*$r,10,35)));
}

// Main function

// We do this because sometime we may need to quit (when we have
// * huge * errors) and we do not want to launch die();

function main($db)
{
  // Time
  $t=time();

  // Possible action of the user
  $do=$_GET["do"];

  // Tables...
  $bidsT=table_prefix."bids";
  $booksT=table_prefix."books";
  $sellsT=table_prefix."sells";
  $usersT=table_prefix."users";
  
  // Define when you need to be logged and when not
  // Login not required for those actions
  $no_login_actions=array("main","create_user","log_in_out","search_books");
  // Login required for those actions
  $login_actions=array("profile","enter_book","sell_book","bid");

  // Menus when logged in and out
  $menu_noLog=array(
          array("Main page","main"),
          array("Register","create_user"),
          array("Log in","log_in_out"),
          array("Search for books","search_books")
        );
  $menu_login=array(
          array("Main page","main"),
          array("Edit profile","profile"),
          array("Sell a book","sell_book"),
          array("Bid for a book","bid"),
          array("Log out","log_in_out")
        );

  // Find the current user_id based on the GET passed session id
  // And also creates the "&sess_id=xxxxx" suffix for the addresses

  // Will put "false" in user and an empty string as the sess_id
  // suffix if no such session exists
  $sess_id = addslashes(stripslashes($_GET['sess_id']));

  if( strlen($sess_id)==32 )
  {

//-- SQL
$query = <<<ENDOFSTRING
SELECT `BUCS_id`, `Forename`, `Surname` FROM $usersT WHERE `Session_id`='$sess_id';
ENDOFSTRING;
//-- ENDSQL

    $result = $db->query($query);
    if(! DB::isError($result) && ($result->numRows()) == 1 )
    {
      $user_id = $result->fetchRow(DB_FETCHMODE_ORDERED);
      $sess_id="&sess_id=$sess_id";
      $user_id=$user_id[0];
    }
    else
    {
      $user_id=false;
      $sess_id="";
    }
  }
  else
  {
    $user_id=false;
    $sess_id="";
  }

  echo "
  <table cellpadding='1' cellspacing='0' width='100%' height='20'> 
      <tr>";

  if($user_id && $do!="log_in_out")
  {
    for($i=0;$i<sizeof($menu_login);$i++)
    {
      echo "<td align='center' valign='middle' class='mainnavbox'><a href=index.php?do=".$menu_login[$i][1]."$sess_id>".$menu_login[$i][0]."</a></td>";
    }
  }
  else
  {
    for($i=0;$i<sizeof($menu_noLog);$i++)
    {
      echo "<td align='center' valign='middle' class='mainnavbox'><a href=index.php?do=".$menu_noLog[$i][1]."$sess_id>".$menu_noLog[$i][0]."</a></td>";
    }
  }

  echo "
    </tr> 
  </table><br>";
  
  echo "
  <table border='0' cellpadding='0' cellspacing='0' width='100%'>
     <tr valign='middle' align='center'>
     <td valign='middle' align='center'>";
        
  // What does the user want to do ?
  if( in_array($do, $no_login_actions) )
  {
    // No login required...
    require_once("$do.php");
  }
  elseif( in_array($do, $login_actions) )
  {
    // Login required!! Check if user logged in
    if($user_id)
    {
      // OK, user logged in... Open page
      require_once("$do.php");
    }
    else
    {
      // User not logged in! Ask for the login
      require_once("log_in_out.php");
    }
  }
  else
  {
    // No action specified: open main page
    require_once("main.php");
  }
  
  echo "
    </td>
    </tr>
  </table>";
}

// Include basic DB information and the PEAR library
if(include(config_address))
{
  $db = DB::connect($dsn);

  if( DB::isError($db) )
  {
    echo "<br><br>".$db->getMessage()."<br><br>";
  }
  else
  {
    main($db);
    $db->disconnect();
  }
}
else
{
  echo "<br><br>Configuration file (".config_address.") cannot be found!";
}
?> 
</td></tr>
<tr><td valign=bottom align=center>
<span class=copy>Project made by <a class=copy href=http://people.bath.ac.uk/st221 target=_blank>Savas Ali Tokmen (st221)</a> and <a class=copy href=mailto:rb247@bath.ac.uk>Rmi Barraquand (rb247)</a><br>
Page optimized for a minimal screen resolution of 1024 x 768, generated in <?echo substr(getmicrotime()-PAGE_START_TIME,0,6);?> seconds<br>&nbsp;</span>
</td></tr></table>

</body>
</html>
<?php

// The script has finished executing... We shall therefore send our buffered
// output to the client, and then kill the script

ob_end_flush();
die(); 
?>